﻿using System;
using System.Collections.Generic;
using System.Text;
using Newtonsoft.Json;
using System.Reflection.Emit;
using System.Reflection;
using System.Linq;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using System.ComponentModel;
using System.Web;
using Newtonsoft.Json.Converters;

namespace Common
{
    public static class Tool
    {
        public static bool IsNull(this object o)
        {
            if (o == null)
            {
                return true;
            }
            if (string.IsNullOrWhiteSpace(o.ToString()))
            {
                return true;
            }
            return false;
        }
        private static Type ExtractGenericInterface(Type queryType, Type interfaceType)
        {
            Func<Type, bool> matchesInterface = t => t.IsGenericType && t.GetGenericTypeDefinition() == interfaceType;
            return (matchesInterface(queryType)) ? queryType : queryType.GetInterfaces().FirstOrDefault(matchesInterface);
        }
        public static int TryToInt(this object o, int faild = -1)
        {
            if (o.IsNull())
            {
                return faild;
            }
            bool result = int.TryParse(o.ToString(), out faild);
            if (!result)
            {
                faild = -1;
            }
            return faild;
        }
        public static string AsSting(this object o)
        {
            if (o != null)
            {
                return o.ToString();
            }
            return string.Empty;
        }
        public static string SerializeIgnore(this object o)
        {
            //使用Newtonsoft 的序列化工具，可以设置忽略循环引用 方法1  
            //但是还有有循环结构数据的转化，只是循环的级数固定了  
            JsonSerializerSettings settings = new JsonSerializerSettings();
            settings.MissingMemberHandling = MissingMemberHandling.Ignore;
            settings.ReferenceLoopHandling = ReferenceLoopHandling.Ignore;
            settings.DateTimeZoneHandling = DateTimeZoneHandling.Utc;
            IsoDateTimeConverter timeFormat = new IsoDateTimeConverter();
            timeFormat.DateTimeFormat = "yyyy-MM-dd HH:mm:ss";
            settings.Converters.Add(timeFormat);
            settings.Converters.Add(new StringEnumConverter());
            //settings.Converters.Add(new BoolConverter());
            settings.Formatting = Formatting.None;
            string result = JsonConvert.SerializeObject(o, settings);
            return result;
        }
        /// <summary>
        /// 将对象序列化为byte[]
        /// 使用IFormatter的Serialize序列化
        /// </summary>
        /// <param name="obj">需要序列化的对象</param>
        /// <returns>序列化获取的二进制流</returns>
        public static string FormatterObjectBytes(this object obj)
        {
            if (obj == null)
                throw new ArgumentNullException("obj");
            byte[] buff;
            try
            {
                using (var ms = new MemoryStream())
                {
                    IFormatter iFormatter = new BinaryFormatter();
                    iFormatter.Serialize(ms, obj);
                    buff = ms.GetBuffer();
                }
            }
            catch (Exception er)
            {
                throw new Exception(er.Message);
            }
            return UTF8Encoding.UTF8.GetString(buff);
        }
        /// <summary>
        /// 将byte[]反序列化为对象
        /// 使用IFormatter的Deserialize发序列化
        /// </summary>
        /// <param name="buff">传入的byte[]</param>
        /// <returns></returns>
        public static T FormatterByteObject<T>(this string str)
        {
            byte[] buff = UTF8Encoding.UTF8.GetBytes(str);
            if (buff == null)
                throw new ArgumentNullException("buff");
            object obj;
            try
            {
                using (var ms = new MemoryStream())
                {
                    IFormatter iFormatter = new BinaryFormatter();
                    obj = iFormatter.Deserialize(ms);
                }
            }
            catch (Exception er)
            {
                throw new Exception(er.Message);
            }
            return (T)obj;
        }
        public static T Deserialize<T>(this string _val)
        {
            if (_val.IsNull())
            {
                return default(T);
            }
            return JsonConvert.DeserializeObject<T>(_val);
        }
        public static Type DynamicClass(IEnumerable<PropertyInfo> ps)
        {
            AssemblyName abName = new AssemblyName("dynamic");
            AssemblyBuilder asb = AppDomain.CurrentDomain.DefineDynamicAssembly(abName, AssemblyBuilderAccess.Run);
            ModuleBuilder mb = asb.DefineDynamicModule(abName.Name);
            TypeBuilder tb = mb.DefineType("NameHelper._dynamicKey");
            var tt = from t in ps select tb.DefineField(t.Name, t.PropertyType, FieldAttributes.Public);
            return tb.CreateType();
        }

        //public static void SetStringRedis(this object o, string key)
        //{
        //    RedisHelper.SetString(key, o);
        //}
        public static string GetConcateStr<T>(this IEnumerable<T> List, Func<T, string> fn)
        {

            StringBuilder sb = new StringBuilder();
            foreach (var item in List)
            {
                sb.Append(fn(item));
                sb.Append(',');
            }
            sb.Remove(sb.Length - 1, 1);
            return sb.ToString();
        }

        public static string GeneratePath<T>(this IEnumerable<string> List, Func<T, string> fn)
        {
            StringBuilder sb = new StringBuilder();
            foreach (var item in List)
            {
            }
            return sb.ToString();
        }
        public static PropertyInfo GetPropertyInfoByType(this object t, string name)
        {
            Type type = t.GetType();
            var prop = type.GetProperty("deleted");
            return prop;
        }

        public static IEnumerable<string> GetAllPropName<T>()
        {
            Type t = typeof(T);
            var propList = t.GetProperties();
            var result = propList.Select(d => d.Name);
            return result;
        }

        public static bool Equeals(this string s, string val2)
        {
            return s.Equals(val2);
        }

        private static T ReferenceBind<T>(object o, object ode, Type tSource, Type dSource, bool ExcludeId = true)
        {
            var sourceProp = tSource.GetProperties();
            var tragetProp = dSource.GetProperties();
            if (ExcludeId)
            {
                sourceProp = sourceProp.Where(d => !d.Name.Equeals("Id")).ToArray();
            }
            foreach (var item in sourceProp)
            {
                var TempResult = tragetProp.Where(d => d.Name.Equeals(item.Name) && d.PropertyType == item.PropertyType);
                if (TempResult != null)
                {
                    int count = TempResult.Count();
                    if (count > 1)
                    {
                        throw new FieldAccessException(string.Format("无法进行值转移,找到多个\t原对象:{0}\t目标对象:{1}", tSource.Name, dSource.Name));
                    }
                    else if (count == 1)
                    {
                        var first = TempResult.First();
                        var propType = first.PropertyType;
                        if (propType.IsClass && (propType != typeof(string)))//如果是类的话,那么继续绑定
                        {
                            ReferenceBind<T>(o,
                                ode,
                                sourceProp.Where(d => d.Name.Equeals(item.Name) && d.PropertyType == item.PropertyType).First().PropertyType,
                                first.PropertyType);
                        }
                        else
                        {
                            if (first.CanWrite)
                            {
                                first.SetValue(ode, item.GetValue(o));
                            }
                        }
                    }
                    //else
                    //{
                    //    throw new FieldAccessException(string.Format("在目标对象中找不到对应属性\t原对象:{0}\t目标对象:{1}", tSource.Name, dSource.Name));
                    //}
                }
            }
            return (T)ode;
        }
        /// <summary>
        /// 将源目标值转移到对应目标对象T中
        /// </summary>
        /// <typeparam name="T">对应类型</typeparam>
        /// <param name="source">原类型</param>
        /// <returns></returns>
        public static T TransferVal<T>(this object source, object de, bool ExcludeId = true)
        {
            Type tSource = source.GetType();//源对象类型
            Type traget = typeof(T);//目标对象类型
            return ReferenceBind<T>(source, de, tSource, traget, ExcludeId);

        }
        /// <summary>
        /// 转移原目标的值到目标对象,不包括属性为类的
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="source"></param>
        /// <param name="de"></param>
        /// <param name="ExcludeId"></param>
        /// <returns></returns>
        public static T TransFerValNotClass<T>(this object source, object de, bool ExcludeId = false)
        {
            if (source == null || de == null)
            {
                throw new ArgumentNullException();
            }
            Type tSource = source.GetType();//源对象类型
            Type traget = typeof(T);//目标对象类型
            var sourceProp = tSource.GetProperties();
            var tragetProp = traget.GetProperties();
            if (ExcludeId)
            {
                sourceProp = sourceProp.Where(d => !d.Name.Equeals("Id")).ToArray();
            }
            foreach (var item in sourceProp)
            {
                var TempResult = tragetProp.Where(d => d.Name.Equeals(item.Name) && d.PropertyType == item.PropertyType);
                if (TempResult != null)
                {
                    int count = TempResult.Count();
                    if (count == 1)
                    {
                        var first = TempResult.First();
                        if (isGene(first))//跳过泛型参数
                        {
                            continue;
                        }
                        first.SetValue(de, item.GetValue(source));
                    }
                }
            }
            return (T)de;
        }
        private static bool isGene(PropertyInfo propInfo)
        {
            return propInfo.PropertyType.IsGenericParameter;
        }
        private static Object ChangeType(object value, Type targetType)
        {
            Type convertType = targetType;
            if (targetType.IsGenericType && targetType.GetGenericTypeDefinition().Equals(typeof(Nullable<>)))
            {
                NullableConverter nullableConverter = new NullableConverter(targetType);
                convertType = nullableConverter.UnderlyingType;
            }
            return Convert.ChangeType(value, convertType);
        }
        /// <summary>
        /// 生成时间戳
        /// </summary>
        /// <returns></returns>
        public static int GetTimeSpan()
        {
            TimeSpan ts = DateTime.UtcNow - new DateTime(1970, 1, 1, 0, 0, 0, 0);
            return (int)ts.TotalSeconds;
        }
        /// <summary>
        /// 生成订单流水号
        /// 生成规则
        /// 1.循环10次得到字母
        /// 2.得到一个时间戳
        /// </summary>
        /// <returns></returns>
        public static string GenerateSerialNumber()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("P");
            sb.Append(DateTime.Now.ToString("yyyyMMddHHmmssf"));
            Random r = new Random();
            sb.Append(r.Next(1000, 9999));
            return sb.ToString();
        }

        public static PageResult GeneratePageResult<T>(this IEnumerable<T> data, PageRequest page, int total)
        {
            PageResult result = new PageResult();
            result.PageSize = page.PageSize;
            result.TotalCount = total;
            result.Current = page.PageIndex;
            result.Data = data;
            return result;
        }
        /// <summary>
        /// 生成指定位数的guid
        /// </summary>
        /// <param name="count"></param>
        /// <returns></returns>
        public static string GenerateGuidByNumber(int count)
        {
            var guid = Guid.NewGuid().ToString().Split('-').Aggregate("", (d1, d2) => { return d1 + d2; });
            try
            {
                return guid.Substring(0, count);
            }
            catch
            {
                return "";
            }
        }
    }

    public class BoolConverter : JsonConverter
    {
        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
            writer.WriteValue(((bool)value) ? "是" : "否");
        }

        public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
        {
            return reader.Value.ToString() == "是";
        }

        public override bool CanConvert(Type objectType)
        {
            return objectType == typeof(bool);
        }
    }
}
