﻿using Dapper;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Configuration;
using System.Data.SqlClient;
using System.Data.Common;

namespace DbKernel
{
    public class SqlProvider
    {
        private const string DATA_PAGE = "prco_DataPage";
        private static string connection = ConfigurationManager.ConnectionStrings["influence"].ToString();
        #region 通用分页
        /// <summary>
        /// 通用数据分页方法
        /// </summary>
        /// <typeparam name="T">实体类型</typeparam>
        /// <param name="TableName">表名</param>
        /// <param name="Fields">字段</param>
        /// <param name="Where">查询条件</param>
        /// <param name="Order">排序条件</param>
        /// <param name="PageSize">每页条数</param>
        /// <param name="PageIndex">当前页码</param>
        /// <param name="TotalRecord">总条数</param>
        public static List<T> DataPage<T>(string TableName, string Fields, string Where, string Order, int PageSize, int PageIndex, out int TotalRecord)
        {
            DynamicParameters param = new DynamicParameters();
            param.Add("@TableName", TableName, DbType.String, ParameterDirection.Input, 50);
            param.Add("@ReFieldsStr", Fields, DbType.String, ParameterDirection.Input, 200);
            param.Add("@OrderString", Order, DbType.String, ParameterDirection.Input, 200);
            param.Add("@WhereString", Where, DbType.String, ParameterDirection.Input, 500);
            param.Add("@PageSize", PageSize, DbType.Int32, ParameterDirection.Input, 4);
            param.Add("@PageIndex", PageIndex, DbType.Int32, ParameterDirection.Input, 4);
            param.Add("@TotalRecord", null, DbType.Int32, ParameterDirection.Output, 4);
            List<T> list = SP_Query<T>(DATA_PAGE, param);
            TotalRecord = param.Get<int>("@TotalRecord");
            return list;
        }
        #endregion

        #region 返回状态
        /// <summary>
        /// 执行存储过程（返回执行状态）
        /// </summary>
        public static int SP_Execute(string sql, DynamicParameters param)
        {
            return SP_Execute(sql, param, CommandType.StoredProcedure);
        }

        /// <summary>
        /// 执行存储过程/SQL（返回执行状态）
        /// </summary>
        public static int SP_Execute(string sql, DynamicParameters param, CommandType type)
        {
            using (IDbConnection conn = OpenConnection())
            {
                return conn.Execute(sql, param, null, null, type);
            }
        }
        /// <summary>
        /// 执行存储过程（返回执行状态）
        /// </summary>
        public static int SP_Execute(string sql, object param)
        {
            return SP_Execute(sql, param, CommandType.StoredProcedure);
        }

        /// <summary>
        /// 执行存储过程/SQL（返回执行状态）
        /// </summary>
        public static int SP_Execute(string sql, object param, CommandType type)
        {
            using (IDbConnection conn = OpenConnection())
            {
                return conn.Execute(sql, param, null, null, type);
            }
        }
        #endregion

        #region 返回实体
        /// <summary>
        /// 执行存储过程（返回一个实体）
        /// </summary>
        public static T SP_QueryFirst<T>(string sql, DynamicParameters param)
        {
            return SP_QueryFirst<T>(sql, param, CommandType.StoredProcedure);
        }
        /// <summary>
        /// 执行存储过程（返回一个实体）
        /// </summary>
        public static T SP_QueryFirst<T>(string sql, object param)
        {
            return SP_QueryFirst<T>(sql, param, CommandType.StoredProcedure);
        }
        /// <summary>
        /// 执行存储过程/SQL（返回一个实体）
        /// </summary>
        public static T SP_QueryFirst<T>(string sql, DynamicParameters param, CommandType type)
        {
            using (IDbConnection conn = OpenConnection())
            {
                return conn.QueryFirstOrDefault<T>(sql, param, null, null, type);
            }
        }
        /// <summary>
        /// 执行存储过程/SQL（返回一个实体）
        /// </summary>
        public static T SP_QueryFirst<T>(string sql, object param, CommandType type)
        {
            using (IDbConnection conn = OpenConnection())
            {
                return conn.QueryFirstOrDefault<T>(sql, param, null, null, type);
            }
        }
        #endregion

        #region 返回List
        /// <summary>
        /// 执行存储过程（返回一个列表）
        /// </summary>
        public static List<T> SP_Query<T>(string sql, DynamicParameters param)
        {
            return SP_Query<T>(sql, param, CommandType.StoredProcedure);
        }

        /// <summary>
        /// 执行存储过程/SQL（返回一个列表）
        /// </summary>
        public static List<T> SP_Query<T>(string sql, DynamicParameters param, CommandType type)
        {
            using (IDbConnection conn = OpenConnection())
            {
                return conn.Query<T>(sql, param, null, false, null, type).AsList();
            }
        }
        /// <summary>
        /// 执行存储过程（返回一个列表）
        /// </summary>
        public static List<T> SP_Query<T>(string sql, object param)
        {
            return SP_Query<T>(sql, param, CommandType.StoredProcedure);
        }

        public static List<T> SP_Query<T>(string sql, object param, CommandType type)
        {
            using (IDbConnection conn = OpenConnection())
            {
                return conn.Query<T>(sql, param, null, false, null, type).AsList();
            }
        }
        #endregion

        #region 打开表库
        /// <summary>
        /// 打开数据库连接
        /// </summary>
        public static IDbConnection OpenConnection()
        {
            DbConnection conn = new SqlConnection(connection);
            if (conn.State != ConnectionState.Open) { conn.Open(); }
            return conn;
        }
        #endregion
    }
}
